precision highp float;

varying vec2 textureCoordinate;
uniform sampler2D inputTexture;
uniform sampler2D inputTexture2;
uniform float skinHue;
uniform float lum;
uniform float beta;

const float offset = 0.02;
float rgb_max(vec3 rgb) {
    return max(rgb.r, max(rgb.g, rgb.b));
}
float rgb_min(vec3 rgb) {
    return min(rgb.r, min(rgb.g, rgb.b));
}
vec3 rgb2hsl(vec3 rgb) {
    float rgbmax = rgb_max(rgb);
    float rgbmin = rgb_min(rgb);
    vec3 hsl = vec3(0.);
    hsl.z = (rgbmax + rgbmin) / 2.;
    if (rgbmax == rgbmin) {
        hsl.y = 0.;
        hsl.x = -1.;
    } else {
        float delta = rgbmax-rgbmin;
        hsl.y = mix(delta/(2.-rgbmax-rgbmin), delta/(rgbmax+rgbmin), step(hsl.z, .5));
        if (delta == 0.) delta = 1.;
        if (rgb.r == rgbmax) hsl.x = (rgb.g-rgb.b)/delta;
        else if (rgb.g == rgbmax) hsl.x = 2. + (rgb.b-rgb.r)/delta;
        else hsl.x = 4.+(rgb.r-rgb.g)/delta;
        hsl.x /= 6.;
        hsl.x = mix(hsl.x+1., hsl.x, step(0., hsl.x));
    }
    return hsl;
}
float hsl_value(float n1, float n2, float hue) {
    if (hue < 0.){
        hue = hue + 6.;
    } else if (hue > 6.){
        hue = hue - 6.;
    }
    if (hue > 4.){
        return n1;
    } else if (hue > 3.){
        return n1 + (n2 - n1)*(4.-hue);
    } else if (hue > 1.){
        return n2;
    } else {
        return n1 + (n2 - n1) * hue;
    }
}
vec3 hsl2rgb(vec3 hsl) {
    vec3 rgb = vec3(0.);
    if (hsl.y == 0.) {
        rgb.r = hsl.z;
        rgb.g = hsl.z;
        rgb.b = hsl.z;
    } else {
        float m2 = mix(hsl.z + hsl.y - hsl.z*hsl.y, hsl.z*(1.+hsl.y), step(hsl.z, .5));
        float m1 = 2.*hsl.z-m2;
        rgb.r = hsl_value(m1, m2, hsl.x * 6. + 2.);
        rgb.g = hsl_value(m1, m2, hsl.x * 6.);
        rgb.b = hsl_value(m1, m2, hsl.x * 6. - 2.);
    }
    return rgb;
}
float rgb2gray(vec3 c){
    return dot(c, vec3(0.299, 0.587, 0.114));
}
float linear(float t, float b, float c, float d) {
    return mix(b, c, clamp(t/d, 0.0, 1.0));
}
//adjust: -1~1
float map_hue(float value, float adjust) {
    float res = value + adjust;
    return mix(res + 1., mix(res - 1., res, step(res, 1.)), step(0., res));
}
//adjust -1~1
float map_saturation(float value, float adjust) {
    return clamp(value * (adjust + 1.), 0., 1.);
}
//adjust -1~1
float map_lightness(float value, float adjust) {
    return mix(value *(adjust + 1.), value + (adjust* (1.-value)), step(0., adjust));
}
vec4 hslproc(vec4 src, vec3 adjust) {
    vec3 hsl = rgb2hsl(src.rgb);
    hsl.x = map_hue(hsl.x, adjust.x);
    hsl.y = map_saturation(hsl.y, adjust.y);
    hsl.z = map_lightness(hsl.z, adjust.z);
    vec4 dest = src;
    dest.rgb = hsl2rgb(hsl);
    return dest;
}
float pixel_difference(vec3 hsl, vec3 params) {
    float diff = 0.;
    float h = mix(hsl.x + 1., hsl.x, step(params.z-1., hsl.x));
    float center = params.y;
    float m = linear(h-params.x, 0.0, 0.2, offset);
    m = linear(h-params.x-offset, m, 1., center-params.x-offset);
    m = linear(h-center, m, 0.2, params.z-offset-center);
    m = linear(h-params.z+offset, m, 0., offset);
    diff = m;
    return diff;
}
void main() {
    vec3 origin = vec3(330./360., 390./360., 450./360.);
    vec4 color = texture2D(inputTexture, textureCoordinate);
    vec4 mask = texture2D(inputTexture2, textureCoordinate);
    vec3 hsl = rgb2hsl(color.rgb);
    float h = mix(hsl.x + 1., hsl.x, step(origin.z - 1., hsl.x));
    float diff = pixel_difference(hsl, origin);
    float lumRange = mix(mix(0., 1., 1.- abs(hsl.y - 1.) * 0.8), .0, 2. * abs(hsl.z-.5) * 0.8);
    vec3 trans = vec3(skinHue, 0.5, 2. * lum);
    vec4 result = hslproc(color, trans);
    result.a = (1. + beta) * lumRange * diff * result.a * rgb2gray(mask.rgb);
    gl_FragColor = result;
}